from cli.tester.server.main_template import generatedClass
from cli.tester.server.pool_template import generatedClass
from cli.tester.server.server_template_old import generatedClass
from cli.tester.server.test_server_post_template import generatedClass

