#!/usr/bin/env python
# coding: utf-8
"""! @brief ZIDS_Object base class."""


class ZIDS_Object():
    """! ZIDS_Object base class for ZIDS_Predictable.
    by default it is not implemented.
    Can be used for declaring variables that needs to shared between predictable objects."""
    pass

    